// If we do not want to include "bcel" finish this.... possibly only extract 
// class constants.

package edu.kit.iti.depgraph;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

public class ClassPool {
	
	public static final int CONSTANT_Class = 7;
	public static final int CONSTANT_Fieldref = 9;
	public static final int CONSTANT_Methodref = 10;
	public static final int CONSTANT_InterfaceMethodref = 11;
	public static final int CONSTANT_String = 8;
	public static final int CONSTANT_Integer = 3;
	public static final int CONSTANT_Float = 4;
	public static final int CONSTANT_Long = 5;
	public static final int CONSTANT_Double = 6;
	public static final int CONSTANT_NameAndType = 12;
	public static final int CONSTANT_Utf8 = 1;
	
	public static final int CONSTANT_NONE = 0;
	
	private String version;
	private Object[] constants;
	private int[] constantTypes;
	private int thisClassIndex;
	

	public ClassPool(InputStream is) throws IOException {
		read(is);
	}
	
	public ClassPool(byte[] bytes) throws IOException {
		read(new ByteArrayInputStream(bytes));
	}
	
	public ClassPool(String fileName) throws IOException {
		FileInputStream inputStream = new FileInputStream(fileName);
		try {
			read(inputStream);
		} finally {
			if(inputStream != null)
				inputStream.close();
		}
	}

	private void read(InputStream inputStream) throws IOException {

		DataInputStream dis = new DataInputStream(inputStream);
        if (dis.readInt() != 0xCAFEBABE) {
            throw new IOException("This is not a class file");
        }
        
        int minor = dis.readUnsignedShort();
        int major = dis.readUnsignedShort();
        version = major + "." + minor;
        
        int count = dis.readUnsignedShort();
        constantTypes = new int[count];
        constants = new Object[count];
        for (int i = 1; i < count; i++) {
        	
        	int tag = dis.readUnsignedByte();
        	
        	// double and long need 2 places.
        	if(tag == CONSTANT_Double || tag == CONSTANT_Long)
        		i ++;
        	
        	constants[i] = readConst(tag, dis);
        	constantTypes[i] = tag;
        }
        
        // access flags.
        dis.readShort();
        
        thisClassIndex = dis.readUnsignedShort();
	}

	private Object readConst(int tag, DataInputStream dis) throws IOException {
		switch (tag) {
		case CONSTANT_Class:
		case CONSTANT_String:

			return dis.readUnsignedShort();
		case CONSTANT_Fieldref:
		case CONSTANT_Methodref:
		case CONSTANT_InterfaceMethodref:
		case CONSTANT_NameAndType:
			return Pair.make(dis.readUnsignedShort(), dis.readUnsignedShort());

		case CONSTANT_Integer:
			return dis.readInt();

		case CONSTANT_Float:
			return dis.readFloat();

		case CONSTANT_Long:
			return dis.readLong();

		case CONSTANT_Double:
			return dis.readDouble();

		case CONSTANT_Utf8:
			return dis.readUTF();

		default:
			throw new IllegalStateException("Invalid tag in constant pool: " + tag);
		}
	}

	public String getVersion() {
		return version;
	}
	
	public int getThisClassIndex() {
		return thisClassIndex;
	}

	public Object getConstant(int i) {
		return constants[i];
	}

	public int getConstantType(int i) {
		return constantTypes[i];
	}
	
	public String getClassName(int index) {
		if(constantTypes[index] != CONSTANT_Class)
			throw new IllegalArgumentException(index + " does not point to a class, but: " + constantTypes[index]);
		
		index = ((Number)constants[index]).intValue();
		
		if(constantTypes[index] != CONSTANT_Utf8)
			throw new IllegalStateException("Constant Pool defect! String expected");
		
		return (String) constants[index];
	}

	public int getCount() {
		return constants.length;
	}

}
