package edu.kit.iti.depgraph;

import java.io.IOException;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.apache.bcel.classfile.ClassParser;
import org.apache.bcel.classfile.Constant;
import org.apache.bcel.classfile.ConstantClass;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.EmptyVisitor;
import org.apache.bcel.classfile.JavaClass;

public class DependencyGraph {

	static Set<String> dependencies = new HashSet<String>();
	static Map<String, Set<String>> packages = new TreeMap<String, Set<String>>();
	
	private static void addDependency(String file, String rawClass) {
		
		int start = 0;
		while(rawClass.charAt(start) == '[') {
			start ++;
			if(rawClass.charAt(start) != '[') {
				if(rawClass.charAt(start) == 'L') {
					// class type array
					start ++;
				} else {
					// primitive array, nothing to record
					return;
				}
			}
		}
		
		rawClass = rawClass.replace('/', '.');
		
		int lastSlash = rawClass.lastIndexOf(".");
		if(lastSlash > 0) {
			String packageName = rawClass.substring(0, lastSlash);
			dependencies.add(String.format("\"%s\" -> \"cluster_%s\"", file, packageName));
		}
		
	}
	

	public static void analyseDependencies(String file) throws IOException {
		ClassParser cp = new ClassParser(file);
		JavaClass javaClass = cp.parse();
		final String className = javaClass.getClassName();
		final ConstantPool cop = javaClass.getConstantPool();
		
		addKnownClass(className);
		
		for (int i = 0; i < cop.getLength(); i++) {
			Constant constant = cop.getConstant(i);
			if(constant != null) {
				constant.accept(new EmptyVisitor() {
					@Override
					public void visitConstantClass(ConstantClass obj) {
						addDependency(className, obj.getBytes(cop));
					}
				});
			}
		}
	}


	private static void addKnownClass(String className) {
		
		int lastSlash = className.lastIndexOf(".");
		String packageName = "";
		String localName = className;
		if(lastSlash > 0) {
			packageName = className.substring(0, lastSlash);
			localName = className.substring(lastSlash + 1);
		}
		
		Set<String> set = packages.get(packageName);
		if(set == null) {
			set = new HashSet<String>();
			packages.put(packageName, set);
		}
		
		set.add(localName);
	}
	
	public static void main(String[] args) throws IOException {
		for (String string : args) {
			analyseDependencies(string);
		}
		
		System.out.println("digraph G {");
		
		for (Entry<String, Set<String>> entry : packages.entrySet()) {
			String packageName = entry.getKey();
			System.out.println("  subgraph \"cluster_" + packageName + "\" {");
			System.out.println("    label = \"" + packageName + "\";");
			for (String localName : entry.getValue()) {
				//System.out.printf("    \"%s.%s\" [label=\"%s\"];\n", packageName, localName, localName);
				System.out.printf("    \"%s.%s\";\n", packageName, localName);
			}
			System.out.println("  }");
		}
		
		for (String string : dependencies) {
			System.out.println("  " + string + ";");
		}
		System.out.println("}");
	}
	
}
